# -*- coding: utf-8 -*-
"""
Enterprise Integrated Channel Degreed Django application initialization.
"""

from django.apps import AppConfig


class DegreedConfig(AppConfig):
    """
    Configuration for the Enterprise Integrated Channel Degreed Django application.
    """
    name = 'integrated_channels.degreed'
    verbose_name = "Enterprise Degreed Integration"
