# Generated by Django 2.2.16 on 2020-09-14 20:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0112_auto_20200914_0926'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisecustomer',
            name='enable_integrated_customer_learner_portal_search',
            field=models.BooleanField(default=True, help_text='Specifies whether a learner for an integrated channel customer can navigate the enterprise learner portal site using the main menu.'),
        ),
        migrations.AddField(
            model_name='historicalenterprisecustomer',
            name='enable_integrated_customer_learner_portal_search',
            field=models.BooleanField(default=True, help_text='Specifies whether a learner for an integrated channel customer can navigate the enterprise learner portal site using the main menu.'),
        ),
    ]
