# Generated by Django 2.2.17 on 2020-11-23 20:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('integrated_channel', '0001_squashed_0007_auto_20190925_0730'),
    ]

    operations = [
        migrations.AddField(
            model_name='learnerdatatransmissionaudit',
            name='subsection_id',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
    ]
