{% load theme_pipeline staticfiles %}
{% load configuration  %}

<!DOCTYPE html>
<!--[if lt IE 7]><html lang="{{ LANGUAGE_CODE|default:"en-us" }}" class="no-js lt-ie9 lt-ie8 lt-ie7"> <![endif]-->
<!--[if IE 7]>   <html lang="{{ LANGUAGE_CODE|default:"en-us" }}" class="no-js lt-ie9 lt-ie8"> <![endif]-->
<!--[if IE 8]>   <html lang="{{ LANGUAGE_CODE|default:"en-us" }}" class="no-js lt-ie9"> <![endif]-->
<!--[if gt IE 8]><!-->
<html lang="{{ LANGUAGE_CODE|default:"en-us" }}" class="no-js"> <!--<![endif]-->
  <head>
  {% block title %}<title>{{ page_title }} | {{ platform_name }}</title>{% endblock %}
  <meta name="viewport" content="width=device-width, initial-scale=1"/>
  <link rel="icon" type="image/x-icon" href="{% favicon_path %}" />
    {% block styles %}
      {% stylesheet 'style-vendor' %}
      {% stylesheet 'style-main-v2' %}
      <link rel="stylesheet" type="text/css" href="{% static 'css/vendor/font-awesome.css' %}"/>
      {# TODO: When we move to be a service, use the webpack loader's `render_bundle`. #}
      <link rel="stylesheet" type="text/css" href="{% static 'enterprise/bundles/main.style.css' %}"/>
    {% endblock %}

    {% block extrastyles %}{% endblock %}

    {% block head %}
      <script type="text/javascript" src="{% static 'common/js/vendor/jquery.js' %}"></script>
    {% endblock %}

    {% block extrahead %}{% endblock %}

    {% include "enterprise/widgets/segment-io-django.html" %}
  </head>

  <body>
    {% block header %}
      <header class="logo-container">
        <h1>
          <a href="{{ LMS_ROOT_URL }}">
            <img class='header-logo' src="{{ logo_url }}" alt="{{ header_logo_alt_text }}" />
          </a>
          {% if tagline %}
            <span id="header-tagline">{{ tagline }}</span>
          {% endif %}
        </h1>
      </header>
      <hr class="pinstripe"/>
    {% endblock %}

    {% block contents %}{% endblock %}
    <script type="text/javascript" src="{% static 'common/js/vendor/bootstrap.bundle.js' %}"></script>
  </body>
</html>
